% Composite block simulation

kappaLayer1=[2.25 0; 0 0.06]; % orthotropic conductivity matrix
kappaLayer2=[0.25 0; 0 0.25]; % isotropic conductivity matrix
alpha =45;% local material orientation angle
ca=cos(2*pi/360*alpha); sa=sin(2*pi/360*alpha);
Rm = [ca, -sa;sa, ca];% local material directions
num_integ_pts=1; % 1-point quadrature
q = 330; % heat source on top surface

[fens,gcells,groups,edge_gcells,edge_groups] = targe2_mesher({...
    ['curve 1 line -40 20 40 20'],...
    ['curve 2 line 40 20 40 40'],...
    ['curve 3 line 40 40 -40 40'],...
    ['curve 4 line -40 40 -40 20'],...
    ['curve 5 line -40 20 -40 0'],...
    ['curve 6 line -40 0 40 0'],...
    ['curve 7 line 40 0 40 20'],...
    ['curve 8 line -40 0 -40 -20'],...
    ['curve 9 line -40 -20 40 -20'],...
    ['curve 10 line 40 -20 40 0'],...
    ['curve 11 line -40 -20 -40 -40'],...
    ['curve 12 line -40 -40 40 -40'],...
    ['curve 13 line 40 -40 40 -20'],...
    ['subregion 1  property 1 ' ...
    '   boundary 1 2 3 4'],...
    ['subregion 2  property 2 '...
    '   boundary 6 7 -1 5 '],...
    ['subregion 3  property 1 ' ...
    '   boundary 9 10 -6 8'],...
    ['subregion 4  property 2 ' ...
    '   boundary 12 13 -9 11'],...
    ['m-ctl-point constant 2.75']
    }, 1.0);


% Layer 1 creation
propLayer1 = property_diffusion(struct('conductivity',kappaLayer1,...
    'source',0));
materLayer1=mater_diffusion(struct('property',propLayer1));
febLayer1 = feblock_diffusion(struct ('mater',materLayer1,...
    'gcells',gcells(groups{1}),...
    'integration_rule',tri_rule(num_integ_pts),'Rm',Rm));

% Layer 2 creation
propLayer2 = property_diffusion(struct('conductivity',kappaLayer2,...
    'source',0));
materLayer2=mater_diffusion(struct('property',propLayer2));
febLayer2 = feblock_diffusion(struct ('mater',materLayer2,...
    'gcells',gcells(groups{2}),...
    'integration_rule',tri_rule(num_integ_pts)));

% Layer 3 uses same material as Layer 1
febLayer3 = feblock_diffusion(struct ('mater',materLayer1,...
    'gcells',gcells(groups{3}),...
    'integration_rule',tri_rule(num_integ_pts),'Rm',Rm));

% Layer 4 uses same material as Layer 2
febLayer4 = feblock_diffusion(struct ('mater',materLayer2,...
    'gcells',gcells(groups{4}),...
    'integration_rule',tri_rule(num_integ_pts)));

geom = field(struct('name',['geom'], 'dim', 2, 'fens',fens));
theta=field(struct('name',['theta'], 'dim', 1, 'nfens',get(geom,'nfens')));

% Create the edgefeb for the top heat source
edgefeb = feblock_diffusion (struct ('mater',materLayer1,...
    'gcells',edge_gcells([edge_groups{[3]}]),...
    'integration_rule',gauss_rule(1,num_integ_pts)));

% Apply a heat source on the top surface
% amb = clone(theta, ['amb']);
% fenids=[fenode_select(fens,struct('box',[40 -40 40 40],...
%     'inflate', 0.01))];
% prescribed=ones(length(fenids),1);
% comp=[];
% val=zeros(length(fenids),1)+0.0;
% amb = set_ebc(amb, fenids, prescribed, comp, val);
% amb = apply_ebc (amb);

% Apply a constant temperature on the left, bottom and right surface
fenids1=[fenode_select(fens,struct('box', [-40 -40 -40 40],...
    'inflate', 0.01))];
fenids2=[fenode_select(fens,struct('box' ,[40 -40 -40 -40],...
    'inflate', 0.01))];
fenids3=[fenode_select(fens,struct('box', [40 40 -40 40],...
    'inflate', 0.01))];
fenids = cat(2, fenids1, fenids2);
fenids = cat(2, fenids, fenids3);
prescribed=ones(length(fenids),1);
comp=[];
val=zeros(length(fenids),1)+200;% hot temperature
theta = set_ebc(theta, fenids, prescribed, comp, val);
theta = numbereqns (theta);

K = start (sparse_sysmat, get(theta, 'neqns'));
K = assemble (K, conductivity(febLayer1, geom, theta));
K = assemble (K, conductivity(febLayer2, geom, theta));
K = assemble (K, conductivity(febLayer3, geom, theta));
K = assemble (K, conductivity(febLayer4, geom, theta));
% K = assemble (K, surface_transfer(edgefeb, geom, theta));
F = start (sysvec, get(theta, 'neqns'));
% F = assemble(F, source_loads(febLayer1, geom, theta));
% F = assemble(F, source_loads(febLayer2, geom, theta));
% F = assemble(F, source_loads(febLayer3, geom, theta));
% F = assemble(F, source_loads(febLayer4, geom, theta));
F = assemble (F, nz_ebc_loads_conductivity(febLayer1, geom, theta));
F = assemble (F, nz_ebc_loads_conductivity(febLayer2, geom, theta));
F = assemble (F, nz_ebc_loads_conductivity(febLayer3, geom, theta));
F = assemble (F, nz_ebc_loads_conductivity(febLayer4, geom, theta));
fi= force_intensity(struct('magn',q));
F = assemble(F, flux_loads(edgefeb, geom, theta,fi));
% F = assemble(F, nz_ebc_loads_surface_transfer(edgefeb, geom, theta));
% F = assemble(F, surface_transfer_loads(edgefeb,geom,theta,amb));
theta = scatter_sysvec(theta, K\F);

% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
T=get(theta,'values');
dcm=data_colormap(struct('range',[min(T),max(T)],'colormap',jet(10)));
colorfield=field(struct ('name', ['colorfield'], 'data',...
    map_data(dcm, T)));
geomT=field(struct ('name', ['geomT'], ...
    'data',[get(geom,'values'), get(theta,'values')/100]));
for i=1:length (gcells)
    draw(gcells(i), gv, struct ('x',geomT, 'u',0*geomT,...
        'colorfield',colorfield, 'shrink',1,'edgecolor','none'));
    draw(gcells(i), gv, struct ('x',geom, 'u',0*geom, ...
        'facecolor','none'));
end
view (2)
%    draw_integration_points(febLayer1, gv, struct ('x',geom,'u',0*geom, ...
%             'theta', theta, 'scale', 25,'color','red'));
 
